
#include <calibinit\dmcalibinit.h>
#include <math.h>
#include "gmlinifile.h"
#include <highgui.h>


//============================================================
//
//============================================================

void AutotestKnownImages()
{
  gml::CIniFile TestsIni;

  // Load ini file
  TestsIni.SetPath("../../testdata/tests.ini");
  if (!TestsIni.ReadFile())
  { 
    fprintf(stderr, "Failed to load tests file!");
    return;
  }

  CvvImage SrcImage,
           ThreshImage,
           GrayImage;

  std::string            sFileName;
  char                   pcKey[100],  pcValueName[100];
  int                    iStatus;
  CvPoint2D32f           pvPoints[100];
  int                    iPointCount;
  CvSize                 ObjectSize;

  // for each test
  for (int i = 0; i < TestsIni.GetValueI("main", "tests"); i++)
  {
    bool bError = false;
    sprintf(pcKey, "test%d", i);

    //fprintf(stderr, " Test %d\n", i);

    // load image
    sFileName = std::string("..\\..\\testdata\\") + TestsIni.GetValue(pcKey, "image");

    fprintf(stderr, " Image %s\n", TestsIni.GetValue(pcKey, "image").c_str());

    if (!SrcImage.Load(sFileName.c_str(), 1))
    {
      fprintf(stderr, "  Failed to load test %d image!\n", i);
      continue;
    }


    //cvColorConvert
    GrayImage.CopyOf(SrcImage, 0);
    ThreshImage.CopyOf(SrcImage, 0);

    // detect
    ObjectSize.width = TestsIni.GetValueI(pcKey, "ObjectWidth");
    ObjectSize.height = TestsIni.GetValueI(pcKey, "ObjectHeight");

    try
    {
    
      iStatus = dmFindChessBoardCornerGuesses( GrayImage.GetImage(), ThreshImage.GetImage(),
                                               NULL, ObjectSize, pvPoints, &iPointCount,
                                               1);
    }
    catch (...)
    {
      fprintf(stderr, "  Test %d generated exception\n", i);
      continue;
    }

    // check the results
    if (iStatus != TestsIni.GetValueI(pcKey, "result"))
    {
      fprintf(stderr, "  Test %d result mismatch %d != %d\n", i, iStatus, 
              TestsIni.GetValueI(pcKey, "result"));
      continue;
    }

    if (iStatus)
    {
      if (iPointCount != TestsIni.GetValueI(pcKey, "points"))
      {
        fprintf(stderr, "  Test %d point number mismatch %d != %d!\n", i, iPointCount, 
                TestsIni.GetValueI(pcKey, "points"));      
        continue;
      }

      for (int j = 0; j < iPointCount; j++)
      {
        CvPoint2D32f vTempPoint;

        sprintf(pcValueName, "point%dx", j);
        vTempPoint.x = TestsIni.GetValueF(pcKey, pcValueName);

        sprintf(pcValueName, "point%dy", j);
        vTempPoint.y = TestsIni.GetValueF(pcKey, pcValueName);

        /*if (fabs(vTempPoint.x - pvPoints[j].x) > 1 ||
            fabs(vTempPoint.y - pvPoints[j].y) > 1)
        {
          fprintf(stderr, "  Test %d point %d mismatch (%g, %g) != (%g, %g)\n", 
                  i, j, pvPoints[j].x, pvPoints[j].y, vTempPoint.x, vTempPoint.y);      

          bError = true;
        } */         
      }
    }

    if (!bError)
      fprintf(stderr, " OK\n");
  }  
}

//============================================================
//
//============================================================

void AutotestImagesLight()
{
  CvvImage SrcImage,
           ThreshImage,
           GrayImage;

  std::string            sFileName;
  int                    iStatus;
  CvPoint2D32f           pvPoints[100];
  int                    iPointCount;
  CvSize                 ObjectSize;

  // for each test
  sFileName.resize(100);

  for (int iSeq = 1; iSeq <= 3; iSeq++)
    for (int i = 0; i < 30; i++)
    {
      sprintf(&sFileName[0], "..\\..\\testdata\\Sequences\\%d\\%07d.bmp", iSeq, i);

      // load image
      if (!SrcImage.Load(sFileName.c_str(), 1))
        continue;

      GrayImage.CopyOf(SrcImage, 0);
      ThreshImage.CopyOf(SrcImage, 0);

      // detect
      ObjectSize.width = 4;
      ObjectSize.height = 5;

      try
      {
    
        iStatus = dmFindChessBoardCornerGuesses( GrayImage.GetImage(), ThreshImage.GetImage(),
                                                 NULL, ObjectSize, pvPoints, &iPointCount, 1);
      }
      catch (...)
      {
        fprintf(stderr, "  Image %d of sequence %d generated exception\n", i, iSeq);
        continue;
      }

      // check the results
      if (!iStatus)
        fprintf(stderr, "  Test %d of sequence %d FAILED\n", i, iSeq);      
      else
        fprintf(stderr, "  Test %d of sequence %d SUCCESS\n", i, iSeq);      
    }
}  


